
#include "stdAfx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////////////////
//						Team Position
//////////////////////////////////////////////////////////////////////////////////

CTeamPosition::CTeamPosition()
{
  m_dHomeBuffer = 0;
  m_type = PT_None;
  m_side = PS_None;
}


void CTeamPosition::SetHome(C2DVector p, float buf, Ptype t, Pside s)
{
  m_Home = p;
  m_dHomeBuffer = buf;

  m_type = t; 
  m_side = s;
}


void CTeamPosition::SetHomeRange(C2DVector center, C2DVector size)
{
  m_HomeRange = RRectangle(center,size);
}


void CTeamPosition::SetMaxRange (C2DVector center, C2DVector size)
{
  m_MaxRange = RRectangle(center,size);
}


//////////////////////////////////////////////////////////////////////////////////
//							Formation Class
//////////////////////////////////////////////////////////////////////////////////

CFormation::CFormation()
{
}


CFormation::~CFormation()
{
}


void CFormation::SetPosition(Pnum num, C2DVector home, float buffer,
			    C2DVector home_center, C2DVector home_size,
			    C2DVector max_center,  C2DVector max_size,
			    Ptype t, Pside s)
{
  /* Use this to put in the parameters for the positions in a given formation */
  PositionList[num].SetHome(home,buffer,t,s);
  PositionList[num].SetHomeRange(home_center,home_size);
  PositionList[num].SetMaxRange(max_center,max_size);
}


CTeamPosition *CFormation::GetPosition(Pnum num)
{
//  if (num > size || num < 1) 
    //my_error("position list index only goes from 1 to size %d (trying to get %d)",size,num);
  return &(PositionList[num]); 
}



Pnum CFormation::LocationToPosition(C2DVector p)
{
  /* Finds the position whose home range (x,y) falls.  If more than one,
     takes the one closest to the home position.  If none, takes the one
     whose home range rectangle is closest */

  double tiebreakDist = -1000;  /* > 2 * (pitch_length + pitch_width) */
  Pnum position;
  RRectangle *HomeRange;
  
  for (int i=1; i<=GetSize(); i++){
    HomeRange = GetPosition(i)->GetHomeRange();
    if ( HomeRange->IsWithin(p) ){  /* Check if (x,y) is within the home range */
      double HomeDist = DistanceToPositionHome(i,p);
      if ( tiebreakDist < 0 || HomeDist < tiebreakDist ){
	position = i;
	tiebreakDist = HomeDist;
      }
    }    /* We're not in the range of this position. Now check if closest so far */
    else if ( tiebreakDist < 0 && HomeRange->DistanceToEdge(p) > tiebreakDist ){
      position = i;
      tiebreakDist = HomeRange->DistanceToEdge(p);    /* a negative number     */
    }
  }

  return position;
}

double CFormation::DistanceToPositionHome(Pnum position, C2DVector p)
{
  C2DVector Home = GetPosition(position)->GetHome();
  return (Home-p).GetDistance();
}


Pnum CFormation::ClosestPosition(C2DVector p)
{
  double dist, minDist = HUGE;
  Pnum closestPosition;
  for (int position=1; position<=m_size; position++){
    dist = DistanceToPositionHome(position,p);
    if (dist < minDist){
      minDist = dist;
      closestPosition = position;
    }
  }
  return closestPosition;
}



KPDL_RESULT CFormation::SetXML(IXMLDOMNodePtr &pNode)
{
	_variant_t vValue;
	IXMLDOMNodeListPtr pChildList;
	IXMLDOMNodePtr pChild;

	//
	// read attributes
	//
	pNode->attributes->getNamedItem("name")->get_nodeValue(&vValue);
	m_name = _bstr_t(vValue);

	//
	// read positions
	//
	pChildList = pNode->selectNodes(L"position");
	pChildList->reset();

	pChildList->get_length( &m_size );

	while((pChild = pChildList->nextNode()) != NULL)
	{
		//cCond.SetXML(pChild);
		//cCondList.push_back(cCond);
		string tmp;
		long pnum, ptype, pside;
		double xhome, yhome;
		
		pChild->attributes->getNamedItem("pid")->get_nodeValue(&vValue);
		pnum = (long)vValue;
		
		pChild->attributes->getNamedItem("x")->get_nodeValue(&vValue);
		tmp = _bstr_t(vValue);
		xhome = atof( tmp.c_str() );

		pChild->attributes->getNamedItem("y")->get_nodeValue(&vValue);
		tmp = _bstr_t(vValue);
		yhome = atof( tmp.c_str() );

		pChild->attributes->getNamedItem("ptype")->get_nodeValue(&vValue);
		ptype = (long)vValue;

		if ( yhome < 0.5 ) pside = 0;
		else if ( yhome > 0.5 ) pside = 2;
		else pside = 1;

		//ajajajaj....

		SetPosition( pnum, C2DVector(xhome,yhome), 10, C2DVector(xhome,yhome), C2DVector(xhome-0.1, yhome+0.1),  C2DVector(xhome,yhome), C2DVector(xhome-0.1, yhome+0.1), (enum Ptype) ptype, (enum Pside) pside );
	}


	return KPDL_SUCCESS;
}
